import Imap from 'imap';
import { simpleParser } from 'mailparser';
import { insertOrUpdateBooking, checkIfEmailProcessed } from './database.js';
import { parseEmail } from './emailParser.js';
import { sendToOpenAI } from './openAIRequests.js';
import { imapConfig } from './config.js';
import chalk from 'chalk';

const imap = new Imap(imapConfig);
let processedEmails = 0;
let skippedEmails = 0;
let errorEmails = 0;

function logSummary() {
    console.log(chalk.green(`\nSummary:`));
    console.log(chalk.green(`Processed Emails: ${processedEmails}`));
    console.log(chalk.yellow(`Skipped Emails: ${skippedEmails}`));
    console.log(chalk.red(`Error Emails: ${errorEmails}`));
}

function checkEmails() {
    imap.once('ready', () => {
        const sinceDate = new Date();
        sinceDate.setDate(sinceDate.getDate() - 2); // Set date to 48 hours ago
        const searchCriteria = ['ALL', ['SINCE', sinceDate.toISOString()]];

        imap.openBox('INBOX', false, async (err, box) => {
            if (err) throw err;
            imap.search(searchCriteria, (err, results) => {
                if (err || !results || !results.length) {
                    console.log('No emails found');
                    imap.end();
                    logSummary();
                    return;
                }

                const f = imap.fetch(results, { bodies: ['HEADER.FIELDS (FROM TO SUBJECT DATE)', 'TEXT'] });
                f.on('message', (msg, seqno) => {
                    const attrs = {};
                    msg.on('attributes', (attribs) => {
                        attrs.uid = attribs.uid;
                    });
                    msg.on('body', async (stream, info) => {
                        try {
                            const parsed = await simpleParser(stream);
                            const alreadyProcessed = await checkIfEmailProcessed(parsed.messageId);
                            if (alreadyProcessed) {
                                console.log(chalk.yellow(`Email already processed: ${parsed.messageId}`));
                                skippedEmails++;
                                return;
                            }
                            const emailData = parseEmail(parsed);
                            if (!emailData) {
                                console.debug(chalk.gray(`Skipping irrelevant or spam email: ${parsed.subject}`));
                                skippedEmails++;
                                return;
                            }

                            const aiPrompt = `
                            Bitte extrahieren Sie die folgenden Buchungsdetails aus der E-Mail im JSON-Format:
{
  "email_id": "<Email ID>",
  "buchungs_status": "<'gebucht' wenn eine verbindliche Buchung vorliegt, ansonsten 'anfrage'>",
  "vorname": "<Vorname des Gastes oder null>",
  "nachname": "<Nachname des Gastes oder null>",
  "adresse": "<Adresse des Gastes oder null>",
  "plz": "<Postleitzahl oder null>",
  "ort": "<Ort oder null>",
  "geburtsdatum": "<Geburtsdatum im Format YYYY-MM-DD oder null>",
  "telefonnummer": "<Telefonnummer oder null>",
  "emailadresse": "<E-Mail-Adresse des Gastes oder null>",
  "firmenname": "<Name der Firma oder null>",
  "anzahl_personen": "<Anzahl der Personen>",
  "anzahl_zimmer": "<Anzahl der Zimmer>",
  "einzelpreis": "<Preis pro Zimmer oder null>",
  "gesamtpreis": "<Gesamtpreis oder null>",
  "zimmernummer": "<Zimmernummer oder null>",
  "buchungs_kommentare": "<Besondere Wünsche oder Kommentare>",
  "email_antwort": "<Antwort auf die E-Mail oder null>",
  "eingecheckt_erledigt": false,
  "ai_comment": "Zusätzliche Anmerkungen zur Buchung oder Anfrage"
}

Stellen Sie sicher, dass Sie im Emailtext wirklich alle Informationen finden und verwenden Sie 'null' für fehlende Informationen. Firmennamen lassen sich ggf. auch erahnen aus der Emailsignatur, etc. Die Daten müssen im richtigen Format vorliegen, insbesondere das Geburtsdatum im Format 'YYYY-MM-DD'. Beachten Sie, ob die E-Mail eine verbindliche Buchung oder nur eine Anfrage ist.

Es ist sicherzustellen, dass im ai comment vermerkt ist, ob es sich um eine verbindliche Buchung oder nur eine Anfrage handelt, und ob eine Antwort gegeben werden muss.`;

                            const aiResponse = await sendToOpenAI(aiPrompt);
                            const parsedAIResponse = JSON.parse(aiResponse);

                            if (parsedAIResponse.email_id && parsedAIResponse.vorname && parsedAIResponse.nachname) {
                                Object.assign(emailData, parsedAIResponse);

                                try {
                                    await insertOrUpdateBooking(emailData);
                                    console.log(chalk.green(`Email processed and saved: ${parsedAIResponse.email_id}`));
                                    console.log(chalk.green(`Booking: ${parsedAIResponse.vorname} ${parsedAIResponse.nachname}, ${parsedAIResponse.anzahl_personen} Personen, ${parsedAIResponse.gesamtpreis}€`));
                                    processedEmails++;
                                } catch (error) {
                                    console.error(chalk.red('Error during database operation:', error));
                                    errorEmails++;
                                }
                            } else {
                                console.error(chalk.red('Error in AI response: Missing required fields.'));
                                errorEmails++;
                            }
                        } catch (error) {
                            if (error.code === 'rate_limit_exceeded') {
                                console.error(chalk.red('Rate limit exceeded, stopping processing:', error.message));
                                logSummary();
                                process.exit(1);
                            } else {
                                console.error(chalk.red('Error during email processing:', error));
                                errorEmails++;
                            }
                        }
                    });
                });
                f.once('end', () => {
                    console.log('Done fetching all messages.');
                    imap.end();
                    logSummary();
                });
            });
        });
    });

    imap.once('error', (err) => console.error(chalk.red('IMAP connection error:', err)));
    imap.once('end', () => console.log('IMAP connection closed'));
    imap.connect();
}

export { checkEmails };
